@@ -2,7 +2,7 @@ import torch
 from torch import nn
 from torch.optim import AdamW
 from tf_bind_transformer.data import read_bed, collate_dl_outputs, get_dataloader, remap_df_add_experiment_target_cell
-from tf_bind_transformer.data import RemapAllPeakDataset, NegativePeakDataset
+from tf_bind_transformer.data import RemapAllPeakDataset, NegativePeakDataset, ScopedNegativePeakDataset
 
 def exists(val):
     return val is not None
@@ -53,7 +53,12 @@ class Trainer(nn.Module):
         shift_aug_range = (-2, 2),
         rc_aug = True,
         experiments_json_path = None,
-        read_value_aux_loss = False
+        read_value_aux_loss = False,
+        checkpoint_filename = './checkpoint.pt',
+        include_scoped_negs = False,
+        scoped_negs_remap_bed_path = None,
+        scoped_negs_path = None,
+        scoped_negs_exts = '.bed.bool.npy'
     ):
         super().__init__()
         self.model = model
@@ -119,9 +124,25 @@ class Trainer(nn.Module):
             experiments_json_path = experiments_json_path
         )
 
+        self.include_scoped_negs = include_scoped_negs
+
         self.dl = get_dataloader(self.ds, cycle_iter = True, shuffle = shuffle, batch_size = batch_size)
         self.neg_dl = get_dataloader(self.neg_ds, cycle_iter = True, shuffle = shuffle, batch_size = batch_size)
 
+        if include_scoped_negs:
+            self.scoped_neg_ds = ScopedNegativePeakDataset(
+                fasta_file = fasta_file,
+                factor_fasta_folder = factor_fasta_folder,
+                numpy_folder_with_scoped_negatives = scoped_negs_path,
+                remap_bed_file = scoped_negs_remap_bed_path,
+                exts = scoped_negs_exts,
+                exclude_targets = held_out_targets,
+                exclude_cell_types = held_out_cell_types,
+                filter_chromosome_ids = train_chromosome_ids
+            )
+
+            self.scoped_neg_dl = get_dataloader(self.scoped_neg_ds, cycle_iter = True, shuffle = shuffle, batch_size = batch_size)
+
         self.valid_dl = get_dataloader(self.valid_ds, cycle_iter = True, shuffle = shuffle, batch_size = batch_size)
         self.valid_neg_dl = get_dataloader(self.valid_neg_ds, cycle_iter = True, shuffle = shuffle, batch_size = batch_size)
 
@@ -135,6 +156,8 @@ class Trainer(nn.Module):
         self.validate_every = validate_every
         self.register_buffer('steps', torch.Tensor([0.]))
 
+        self.checkpoint_filename = checkpoint_filename
+
     def forward(
         self,
         finetune_enformer_ln_only = True,
@@ -145,7 +168,12 @@ class Trainer(nn.Module):
 
         total_train_loss = 0
         for _ in range(self.grad_accum_every):
-            seq, tf_aa, contextual_texts, peaks_nr, read_value, binary_target = collate_dl_outputs(next(self.dl), next(self.neg_dl))
+            dl_outputs = [next(self.dl), next(self.neg_dl)]
+
+            if self.include_scoped_negs:
+                dl_outputs.append(next(self.scoped_neg_dl))
+
+            seq, tf_aa, contextual_texts, peaks_nr, read_value, binary_target = collate_dl_outputs(*dl_outputs)
             seq, binary_target, read_value, peaks_nr = seq.cuda(), binary_target.cuda(), read_value.cuda(), peaks_nr.cuda()
 
             loss, aux_loss = self.model(
@@ -173,7 +201,7 @@ class Trainer(nn.Module):
         self.optim.step()
         self.optim.zero_grad()
 
-        if not (curr_step % self.validate_every):
+        if (curr_step % self.validate_every) == 0:
             self.model.eval()
 
             total_valid_loss = 0
@@ -207,7 +235,8 @@ class Trainer(nn.Module):
             print(f'{curr_step} valid loss: {avg_valid_loss}')
             print(f'{curr_step} valid accuracy: {avg_valid_accuracy}')
 
-            torch.save(self.model.state_dict(), f'./checkpoint.pt')
+            if curr_step > 0:
+                torch.save(self.model.state_dict(), self.checkpoint_filename)
 
         self.steps += 1
         return logs
